@@ -82,6 +82,7 @@ pkgadminfile(struct pkginfo *pkg, const char *filetype)
 
   varbuf_reset(&vb);
   varbuf_add_str(&vb, infodir);
+  varbuf_add_char(&vb, '/');
   varbuf_add_str(&vb, pkg->name);
   varbuf_add_char(&vb, '.');
   varbuf_add_str(&vb, filetype);
@@ -276,7 +277,7 @@ pkg_files_add_file(struct pkginfo *pkg, const char *filename,
 void
 ensure_packagefiles_available(struct pkginfo *pkg)
 {
-  int fd;
+  static int fd;
   const char *filelistfile;
   struct fileinlist **lendp;
   struct stat stat_buf;
@@ -492,12 +493,12 @@ void ensure_allinstfiles_available_quiet(void) {
 }
 
 /*
- * If leaveout is nonzero, will not write any file whose filenamenode
- * has the fnnf_elide_other_lists flag set.
+ * If mask is nonzero, will not write any file whose filenamenode
+ * has any flag bits set in mask.
  */
 void
 write_filelist_except(struct pkginfo *pkg, struct fileinlist *list,
-                      bool leaveout)
+                      enum fnnflags mask)
 {
   static struct varbuf newvb;
   const char *listfile;
@@ -513,9 +514,9 @@ write_filelist_except(struct pkginfo *pkg, struct fileinlist *list,
   file= fopen(newvb.buf,"w+");
   if (!file)
     ohshite(_("unable to create updated files list file for package %s"),pkg->name);
-  push_cleanup(cu_closefile, ehflag_bombout, NULL, 0, 1, (void *)file);
+  push_cleanup(cu_closestream, ehflag_bombout, NULL, 0, 1, (void *)file);
   while (list) {
-    if (!(leaveout && (list->namenode->flags & fnnf_elide_other_lists))) {
+    if (!(mask && (list->namenode->flags & mask))) {
       fputs(list->namenode->name,file);
       putc('\n',file);
     }
