@@ -37,19 +37,23 @@
 #include <dpkg/dpkg.h>
 #include <dpkg/dpkg-db.h>
 #include <dpkg/dir.h>
-#include <dpkg/myopt.h>
+#include <dpkg/options.h>
 #include <dpkg/triglib.h>
 
+#include "infodb.h"
 #include "filesdb.h"
 #include "main.h"
 
+/*
+ * pkgdepcheck may be a virtual pkg.
+ */
 static void checkforremoval(struct pkginfo *pkgtoremove,
-                            struct pkginfo *pkgdepcheck, /* may be virtual pkg */
+                            struct pkginfo *pkgdepcheck,
                             int *rokp, struct varbuf *raemsgs) {
   struct deppossi *possi;
   struct pkginfo *depender;
   int before, ok;
-  
+
   for (possi = pkgdepcheck->installed.depended; possi; possi = possi->rev_next) {
     if (possi->up->type != dep_depends && possi->up->type != dep_predepends) continue;
     depender= possi->up->up;
@@ -79,7 +83,7 @@ void deferred_remove(struct pkginfo *pkg) {
   struct dependency *dep;
 
   debug(dbg_general,"deferred_remove package %s",pkg->name);
-  
+
   if (pkg->status == stat_notinstalled) {
     warning(_("ignoring request to remove %.250s which isn't installed."),
             pkg->name);
@@ -87,7 +91,7 @@ void deferred_remove(struct pkginfo *pkg) {
     return;
   } else if (!f_pending &&
              pkg->status == stat_configfiles &&
-             cipaction->arg != act_purge) {
+             cipaction->arg_int != act_purge) {
     warning(_("ignoring request to remove %.250s, only the config\n"
               " files of which are on the system. Use --purge to remove them too."),
             pkg->name);
@@ -100,7 +104,7 @@ void deferred_remove(struct pkginfo *pkg) {
                 " it should not be removed."));
 
   if (!f_pending)
-    pkg->want= (cipaction->arg == act_purge) ? want_purge : want_deinstall;
+    pkg->want = (cipaction->arg_int == act_purge) ? want_purge : want_deinstall;
   if (!f_noact) modstatdb_note(pkg);
 
   debug(dbg_general,"checking dependencies for remove `%s'",pkg->name);
@@ -119,13 +123,13 @@ void deferred_remove(struct pkginfo *pkg) {
     return;
   } else if (rok == 0) {
     sincenothing= 0;
-    varbufaddc(&raemsgs,0);
+    varbuf_end_str(&raemsgs);
     fprintf(stderr,
             _("dpkg: dependency problems prevent removal of %s:\n%s"),
             pkg->name, raemsgs.buf);
     ohshit(_("dependency problems - not removing"));
   } else if (raemsgs.used) {
-    varbufaddc(&raemsgs,0);
+    varbuf_end_str(&raemsgs);
     fprintf(stderr,
             _("dpkg: %s: dependency problems, but removing anyway as you requested:\n%s"),
             pkg->name, raemsgs.buf);
@@ -149,7 +153,7 @@ void deferred_remove(struct pkginfo *pkg) {
   }
 
   oldconffsetflags(pkg->installed.conffiles);
-    
+
   printf(_("Removing %s ...\n"),pkg->name);
   log_action("remove", pkg);
   trig_activate_packageprocessing(pkg);
@@ -164,7 +168,8 @@ void deferred_remove(struct pkginfo *pkg) {
     maintainer_script_installed(pkg, PRERMFILE, "pre-removal",
                                 "remove", NULL);
 
-    pkg->status= stat_unpacked; /* Will turn into halfinstalled soon ... */
+    /* Will turn into ‘half-installed’ soon ... */
+    pkg->status = stat_unpacked;
   }
 
   removal_bulk(pkg);
@@ -179,20 +184,32 @@ static void push_leftover(struct fileinlist **leftoverp,
   *leftoverp= newentry;
 }
 
-static void removal_bulk_remove_files(
-    struct pkginfo *pkg, 
-    bool *out_foundpostrm)
+static void
+removal_bulk_remove_file(const char *filename, const char *filetype)
+{
+  /* We need the postrm and list files for --purge. */
+  if (strcmp(filetype, LISTFILE) == 0 ||
+      strcmp(filetype, POSTRMFILE) == 0)
+    return;
+
+  debug(dbg_stupidlyverbose, "removal_bulk info not postrm or list");
+
+  if (unlink(filename))
+    ohshite(_("unable to delete control info file `%.250s'"), filename);
+
+  debug(dbg_scripts, "removal_bulk info unlinked %s", filename);
+}
+
+static void
+removal_bulk_remove_files(struct pkginfo *pkg)
 {
   int before;
-  int infodirbaseused;
   struct reversefilelistiter rlistit;
   struct fileinlist *leftover;
   struct filenamenode *namenode;
   static struct varbuf fnvb;
-  DIR *dsd;
-  struct dirent *de;
   struct stat stab;
-  
+
     pkg->status= stat_halfinstalled;
     modstatdb_note(pkg);
     push_checkpoint(~ehflag_bombout, ehflag_normaltidy);
@@ -212,36 +229,40 @@ static void removal_bulk_remove_files(
       usenode = namenodetouse(namenode, pkg);
       trig_file_activate(usenode, pkg);
 
-      varbufreset(&fnvb);
-      varbufaddstr(&fnvb,instdir);
-      varbufaddstr(&fnvb, usenode->name);
+      varbuf_reset(&fnvb);
+      varbuf_add_str(&fnvb, instdir);
+      varbuf_add_str(&fnvb, usenode->name);
       before= fnvb.used;
-      
-      varbufaddstr(&fnvb,DPKGTEMPEXT);
-      varbufaddc(&fnvb,0);
+
+      varbuf_add_str(&fnvb, DPKGTEMPEXT);
+      varbuf_end_str(&fnvb);
       debug(dbg_eachfiledetail, "removal_bulk cleaning temp `%s'", fnvb.buf);
-      
+
       ensure_pathname_nonexisting(fnvb.buf);
-      
+
       varbuf_trunc(&fnvb, before);
-      varbufaddstr(&fnvb,DPKGNEWEXT);
-      varbufaddc(&fnvb,0);
+      varbuf_add_str(&fnvb, DPKGNEWEXT);
+      varbuf_end_str(&fnvb);
       debug(dbg_eachfiledetail, "removal_bulk cleaning new `%s'", fnvb.buf);
       ensure_pathname_nonexisting(fnvb.buf);
-      
+
       varbuf_trunc(&fnvb, before);
-      varbufaddc(&fnvb,0);
+      varbuf_end_str(&fnvb);
       if (!stat(fnvb.buf,&stab) && S_ISDIR(stab.st_mode)) {
         debug(dbg_eachfiledetail, "removal_bulk is a directory");
         /* Only delete a directory or a link to one if we're the only
-         * package which uses it.  Other files should only be listed
-         * in this package (but we don't check).
-         */
-	if (hasdirectoryconffiles(namenode,pkg)) {
+         * package which uses it. Other files should only be listed
+         * in this package (but we don't check). */
+        if (dir_has_conffiles(namenode, pkg)) {
 	  push_leftover(&leftover,namenode);
 	  continue;
 	}
-	if (isdirectoryinuse(namenode,pkg)) continue;
+        if (dir_is_used_by_pkg(namenode, pkg, leftover)) {
+          push_leftover(&leftover, namenode);
+          continue;
+        }
+        if (dir_is_used_by_others(namenode, pkg))
+          continue;
       }
       debug(dbg_eachfiledetail, "removal_bulk removing `%s'", fnvb.buf);
       if (!rmdir(fnvb.buf) || errno == ENOENT || errno == ELOOP) continue;
@@ -256,6 +277,11 @@ static void removal_bulk_remove_files(
                 pkg->name, namenode->name, strerror(errno));
         push_leftover(&leftover,namenode);
         continue;
+      } else if (errno == EINVAL && strcmp(usenode->name, "/.") == 0) {
+        debug(dbg_eachfiledetail, "removal_bulk '%s' root directory, cannot remove",
+              fnvb.buf);
+        push_leftover(&leftover, namenode);
+        continue;
       }
       if (errno != ENOTDIR) ohshite(_("cannot remove `%.250s'"),fnvb.buf);
       debug(dbg_eachfiledetail, "removal_bulk unlinking `%s'", fnvb.buf);
@@ -265,45 +291,15 @@ static void removal_bulk_remove_files(
     write_filelist_except(pkg,leftover,0);
     maintainer_script_installed(pkg, POSTRMFILE, "post-removal",
                                 "remove", NULL);
-    varbufreset(&fnvb);
-    varbufaddstr(&fnvb, pkgadmindir());
-    infodirbaseused= fnvb.used;
-    varbufaddc(&fnvb,0);
-    dsd= opendir(fnvb.buf); if (!dsd) ohshite(_("cannot read info directory"));
-    push_cleanup(cu_closedir, ~0, NULL, 0, 1, (void *)dsd);
-    *out_foundpostrm = false;
+
+    trig_parse_ci(pkgadminfile(pkg, TRIGGERSCIFILE),
+                  trig_cicb_interest_delete, NULL, pkg);
+    trig_file_interests_save();
 
     debug(dbg_general, "removal_bulk cleaning info directory");
+    pkg_infodb_foreach(pkg, removal_bulk_remove_file);
+    dir_sync_path(pkgadmindir());
 
-    while ((de = readdir(dsd)) != NULL) {
-    char *p;
-
-      debug(dbg_veryverbose, "removal_bulk info file `%s'", de->d_name);
-      if (de->d_name[0] == '.') continue;
-      p= strrchr(de->d_name,'.'); if (!p) continue;
-      if (strlen(pkg->name) != (size_t)(p-de->d_name) ||
-          strncmp(de->d_name,pkg->name,p-de->d_name)) continue;
-      debug(dbg_stupidlyverbose, "removal_bulk info this pkg");
-      /* We need the postrm and list files for --purge. */
-      if (!strcmp(p+1,LISTFILE)) continue;
-      if (!strcmp(p + 1, POSTRMFILE)) {
-        *out_foundpostrm = true;
-        continue;
-      }
-      debug(dbg_stupidlyverbose, "removal_bulk info not postrm or list");
-      varbuf_trunc(&fnvb, infodirbaseused);
-      varbufaddstr(&fnvb,de->d_name);
-      varbufaddc(&fnvb,0);
-      if (unlink(fnvb.buf))
-        ohshite(_("unable to delete control info file `%.250s'"),fnvb.buf);
-      debug(dbg_scripts, "removal_bulk info unlinked %s",fnvb.buf);
-    }
-
-    /* Sync the info database directory. */
-    dir_sync(dsd, fnvb.buf);
-
-    pop_cleanup(ehflag_normaltidy); /* closedir */
-    
     pkg->status= stat_configfiles;
     pkg->installed.essential = false;
     modstatdb_note(pkg);
@@ -317,6 +313,9 @@ static void removal_bulk_remove_leftover_dirs(struct pkginfo *pkg) {
   static struct varbuf fnvb;
   struct stat stab;
 
+  /* We may have modified this previously. */
+  ensure_packagefiles_available(pkg);
+
   modstatdb_note(pkg);
   push_checkpoint(~ehflag_bombout, ehflag_normaltidy);
 
@@ -328,29 +327,32 @@ static void removal_bulk_remove_leftover_dirs(struct pkginfo *pkg) {
     debug(dbg_eachfile, "removal_bulk `%s' flags=%o",
           namenode->name, namenode->flags);
     if (namenode->flags & fnnf_old_conff) {
-      push_leftover(&leftover,namenode);
+      /* This can only happen if removal_bulk_remove_configfiles() got
+       * interrupted half way. */
+      debug(dbg_eachfiledetail, "removal_bulk expecting only left over dirs, "
+                                "ignoring conffile '%s'", namenode->name);
       continue;
     }
 
     usenode = namenodetouse(namenode, pkg);
     trig_file_activate(usenode, pkg);
 
-    varbufreset(&fnvb);
-    varbufaddstr(&fnvb,instdir);
-    varbufaddstr(&fnvb, usenode->name);
-    varbufaddc(&fnvb,0);
+    varbuf_reset(&fnvb);
+    varbuf_add_str(&fnvb, instdir);
+    varbuf_add_str(&fnvb, usenode->name);
+    varbuf_end_str(&fnvb);
 
     if (!stat(fnvb.buf,&stab) && S_ISDIR(stab.st_mode)) {
       debug(dbg_eachfiledetail, "removal_bulk is a directory");
       /* Only delete a directory or a link to one if we're the only
-       * package which uses it.  Other files should only be listed
-       * in this package (but we don't check).
-       */
-      if (hasdirectoryconffiles(namenode,pkg)) {
-	push_leftover(&leftover,namenode);
-	continue;
+       * package which uses it. Other files should only be listed
+       * in this package (but we don't check). */
+      if (dir_is_used_by_pkg(namenode, pkg, leftover)) {
+        push_leftover(&leftover, namenode);
+        continue;
       }
-      if (isdirectoryinuse(namenode,pkg)) continue;
+      if (dir_is_used_by_others(namenode, pkg))
+        continue;
     }
 
     debug(dbg_eachfiledetail, "removal_bulk removing `%s'", fnvb.buf);
@@ -366,9 +368,23 @@ static void removal_bulk_remove_leftover_dirs(struct pkginfo *pkg) {
               pkg->name, namenode->name, strerror(errno));
       push_leftover(&leftover,namenode);
       continue;
+    } else if (errno == EINVAL && strcmp(usenode->name, "/.") == 0) {
+      debug(dbg_eachfiledetail, "removal_bulk '%s' root directory, cannot remove",
+            fnvb.buf);
+      push_leftover(&leftover, namenode);
+      continue;
     }
     if (errno != ENOTDIR) ohshite(_("cannot remove `%.250s'"),fnvb.buf);
 
+    if (lstat(fnvb.buf, &stab) == 0 && S_ISLNK(stab.st_mode)) {
+      debug(dbg_eachfiledetail, "removal_bulk is a symlink to a directory");
+
+      if (unlink(fnvb.buf))
+        ohshite(_("cannot remove '%.250s'"), fnvb.buf);
+
+      continue;
+    }
+
     push_leftover(&leftover,namenode);
     continue;
   }
@@ -393,18 +409,18 @@ static void removal_bulk_remove_configfiles(struct pkginfo *pkg) {
     printf(_("Purging configuration files for %s ...\n"),pkg->name);
     log_action("purge", pkg);
     trig_activate_packageprocessing(pkg);
-    ensure_packagefiles_available(pkg); /* We may have modified this above. */
+
+    /* We may have modified this above. */
+    ensure_packagefiles_available(pkg);
 
     /* We're about to remove the configuration, so remove the note
-     * about which version it was ...
-     */
+     * about which version it was ... */
     blankversion(&pkg->configversion);
     modstatdb_note(pkg);
-    
+
     /* Remove from our list any conffiles that aren't ours any more or
      * are involved in diversions, except if we are the package doing the
-     * diverting.
-     */
+     * diverting. */
     for (lconffp = &pkg->installed.conffiles; (conff = *lconffp) != NULL; ) {
       for (searchfile= pkg->clientdata->files;
            searchfile && strcmp(searchfile->namenode->name,conff->name);
@@ -426,29 +442,29 @@ static void removal_bulk_remove_configfiles(struct pkginfo *pkg) {
       }
     }
     modstatdb_note(pkg);
-    
+
     for (conff= pkg->installed.conffiles; conff; conff= conff->next) {
     static struct varbuf fnvb, removevb;
       if (conff->obsolete) {
 	debug(dbg_conffdetail, "removal_bulk conffile obsolete %s",
 	      conff->name);
       }
-      varbufreset(&fnvb);
+      varbuf_reset(&fnvb);
       r= conffderef(pkg, &fnvb, conff->name);
       debug(dbg_conffdetail, "removal_bulk conffile `%s' (= `%s')",
             conff->name, r == -1 ? "<r==-1>" : fnvb.buf);
       if (r == -1) continue;
-      conffnameused= fnvb.used-1;
+      conffnameused = fnvb.used;
       if (unlink(fnvb.buf) && errno != ENOENT && errno != ENOTDIR)
         ohshite(_("cannot remove old config file `%.250s' (= `%.250s')"),
                 conff->name, fnvb.buf);
       p= strrchr(fnvb.buf,'/'); if (!p) continue;
       *p = '\0';
-      varbufreset(&removevb);
-      varbufaddstr(&removevb,fnvb.buf);
-      varbufaddc(&removevb,'/');
+      varbuf_reset(&removevb);
+      varbuf_add_str(&removevb, fnvb.buf);
+      varbuf_add_char(&removevb, '/');
       removevbbase= removevb.used;
-      varbufaddc(&removevb,0);
+      varbuf_end_str(&removevb);
       dsd= opendir(removevb.buf);
       if (!dsd) {
         int e=errno;
@@ -486,7 +502,8 @@ static void removal_bulk_remove_configfiles(struct pkginfo *pkg) {
         continue;
       yes_remove:
         varbuf_trunc(&removevb, removevbbase);
-        varbufaddstr(&removevb,de->d_name); varbufaddc(&removevb,0);
+        varbuf_add_str(&removevb, de->d_name);
+        varbuf_end_str(&removevb);
         debug(dbg_conffdetail, "removal_bulk conffile dsd entry removing `%s'",
               removevb.buf);
         if (unlink(removevb.buf) && errno != ENOENT && errno != ENOTDIR)
@@ -494,50 +511,39 @@ static void removal_bulk_remove_configfiles(struct pkginfo *pkg) {
                   removevb.buf, conff->name);
       }
       pop_cleanup(ehflag_normaltidy); /* closedir */
-    
     }
-    
+
+    /* Remove the conffiles from the file list file. */
+    write_filelist_except(pkg, pkg->clientdata->files, fnnf_old_conff);
+
     pkg->installed.conffiles = NULL;
     modstatdb_note(pkg);
-        
+
     maintainer_script_installed(pkg, POSTRMFILE, "post-removal",
                                 "purge", NULL);
 }
 
+/*
+ * This is used both by deferred_remove() in this file, and at the end of
+ * process_archive() in archives.c if it needs to finish removing a
+ * conflicting package.
+ */
 void removal_bulk(struct pkginfo *pkg) {
-  /* This is used both by deferred_remove in this file, and at
-   * the end of process_archive in archives.c if it needs to finish
-   * removing a conflicting package.
-   */
-
-  int pkgnameused;
-  bool foundpostrm = false;
-  const char *postrmfilename;
+  bool foundpostrm;
 
   debug(dbg_general,"removal_bulk package %s",pkg->name);
 
   if (pkg->status == stat_halfinstalled || pkg->status == stat_unpacked) {
-
-    removal_bulk_remove_files(pkg, &foundpostrm);
-
-  } else {
-    struct stat stab;
-
-    postrmfilename= pkgadminfile(pkg,POSTRMFILE);
-    if (!lstat(postrmfilename, &stab))
-      foundpostrm = true;
-    else if (errno == ENOENT)
-      foundpostrm = false;
-    else ohshite(_("unable to check existence of `%.250s'"),postrmfilename);
-
+    removal_bulk_remove_files(pkg);
   }
 
+  foundpostrm = pkg_infodb_has_file(pkg, POSTRMFILE);
+
   debug(dbg_general, "removal_bulk purging? foundpostrm=%d",foundpostrm);
-  
+
   if (!foundpostrm && !pkg->installed.conffiles) {
     /* If there are no config files and no postrm script then we
-     * go straight into `purge'.
-     */
+     * go straight into ‘purge’.  */
     debug(dbg_general, "removal_bulk no postrm, no conffiles, purging");
     pkg->want= want_purge;
 
@@ -548,41 +554,35 @@ void removal_bulk(struct pkginfo *pkg) {
 
   }
 
+  /* I.e., either of the two branches above. */
   if (pkg->want == want_purge) {
-    /* ie, either of the two branches above. */
-    static struct varbuf fnvb;
+    const char *filename;
 
-    /* retry empty directories, and warn on any leftovers that aren't */
+    /* Retry empty directories, and warn on any leftovers that aren't. */
     removal_bulk_remove_leftover_dirs(pkg);
 
-    varbufreset(&fnvb);
-    varbufaddstr(&fnvb, pkgadmindir());
-    varbufaddstr(&fnvb,pkg->name);
-    pkgnameused= fnvb.used;
-    
-    varbufaddstr(&fnvb,"." LISTFILE);
-    varbufaddc(&fnvb,0);
-    debug(dbg_general, "removal_bulk purge done, removing list `%s'",fnvb.buf);
-    if (unlink(fnvb.buf) && errno != ENOENT) ohshite(_("cannot remove old files list"));
-    
-    varbuf_trunc(&fnvb, pkgnameused);
-    varbufaddstr(&fnvb,"." POSTRMFILE);
-    varbufaddc(&fnvb,0);
-    debug(dbg_general, "removal_bulk purge done, removing postrm `%s'",fnvb.buf);
-    if (unlink(fnvb.buf) && errno != ENOENT) ohshite(_("can't remove old postrm script"));
+    filename = pkgadminfile(pkg, LISTFILE);
+    debug(dbg_general, "removal_bulk purge done, removing list `%s'",
+          filename);
+    if (unlink(filename) && errno != ENOENT)
+      ohshite(_("cannot remove old files list"));
+
+    filename = pkgadminfile(pkg, POSTRMFILE);
+    debug(dbg_general, "removal_bulk purge done, removing postrm `%s'",
+          filename);
+    if (unlink(filename) && errno != ENOENT)
+      ohshite(_("can't remove old postrm script"));
 
     pkg->status= stat_notinstalled;
     pkg->want = want_unknown;
 
     /* This will mess up reverse links, but if we follow them
-     * we won't go back because pkg->status is stat_notinstalled.
-     */
-    blankpackageperfile(&pkg->installed);
+     * we won't go back because pkg->status is stat_notinstalled. */
+    pkgbin_blank(&pkg->installed);
   }
-      
+
   pkg->eflag = eflag_ok;
   modstatdb_note(pkg);
 
   debug(dbg_general, "removal done");
 }
-
